/*
  動作
    命令は、アルファベット3文字とする。
    命令とパラメーターの間はスペースで区切る。
    命令の最後は、CR/LF/CR+LFのいずれかを使用する。

    命令表
    stt   測定開始(start) 0.5秒間のカウンタ値を出力し続ける
    stp   測定終了(stop)
    ver   ファームウェアのバージョン
    nam   マシン名
    opt   校正データ等の情報
    stt   データ出力形式(タイミング等の)
    nop   何もしない

    現状
    sttt
      測定を開始する。0.5秒毎に、0.5秒間のカウント値を、出力する。
    stp
      測定を終了
  プログラム
    bluetoothからの1byte受信(BtComm::read)のところで、新しいデータが
    取得できる場合に取得し、そのデータを送信するようにしている。
    これは、マルチスレッドがないので、しょうがない。
*/

//タイマー割り込み関連
#include <avr/interrupt.h>
#include <avr/io.h>
#define INIT_TIMER_COUNT 6
#define RESET_TIMER2 TCNT2 = INIT_TIMER_COUNT  //マクロ
int IntervalTimerCounter = 0;
bool inMeasure = false ;

//外部IO割り込みカウンタ
int GeigerCounter = 0 ;//ガイガーカウンタ
int LastGeigerCounter = 0 ;//最後のガイガーカウンタ
int MeasureCount = 0 ;//測定回数
int LastMeasureCount = 0 ;//測定

//ソフトウェアシリアル関連
#include <SoftwareSerial.h>
#define rxPin 1
#define txPin 7
#define LedPin 5 //or 5


//BluetoothModule
#define HARD_ORIGINAL_BAUD 9600
#define HARD_ORIGINAL_BAUD_STR "9600"
#define GEIGER_BAUD 115200
#define GEIGER_BAUD_STR "115200"
//#define GEIGER_BAUD 57600
//#define GEIGER_BAUD_STR "57600"
#define MACHINE_NAME "CaGeiger"



#include "BtComm.h"

BtComm btComm = BtComm() ;
// set up a new serial port
SoftwareSerial dbgLog =  SoftwareSerial(rxPin, txPin);

int strCmp( char* dist, char* src );
void replyPrint( char* str );
void replyPrint( char* title, char* str );
void setupBluetoothModule( void );
void toScanMode();
bool confirmBtConnection();
void initDebug();
void initTimerCounter();
void changeBaudrete(long a);


//タイマー割り込み
// Aruino runs at 16 Mhz, so we have 1000 Overflows per second...
// 1/ ((16000000 / 64) / 256) = 1 / 1000
ISR(TIMER2_OVF_vect) {
    RESET_TIMER2;
    IntervalTimerCounter ++ ;
    if (IntervalTimerCounter == 500 ) {//500msec
        IntervalTimerCounter = 0 ;
        LastGeigerCounter = GeigerCounter ;
        GeigerCounter = 0 ;
        MeasureCount ++ ;
    }
};
//外部IO割り込み
void CountInterruptFunc()
{
    GeigerCounter ++ ;
}

void initDebug()
{
    pinMode(txPin, OUTPUT);
    dbgLog.begin(9600l);
    dbgLog.println("debug start");
}
void initTimerCounter()
{
    //カウンタの初期化
    IntervalTimerCounter = 0 ;
    MeasureCount = 0 ;
    LastMeasureCount = 0 ;
    inMeasure = false ;
    GeigerCounter = 0 ;
    LastGeigerCounter = 0 ;

    //タイマー割り込み設定
    //Timer2 Settings: Timer Prescaler /64,
    TCCR2B |= (1<<CS22); // turn on CS22 bit
    TCCR2B &= ~((1<<CS21) | (1<<CS20)); // turn off CS21 and CS20 bits
    // Use normal mode
    TCCR2A &= ~((1<<WGM21) | (1<<WGM20)); // turn off WGM21 and WGM20 bits
    // Use internal clock - external clock not used in Arduino
    ASSR |= (0<<AS2);
    TIMSK2 |= (1<<TOIE2) | (0<<OCIE2A); //Timer2 Overflow Interrupt Enable

    RESET_TIMER2;

    sei();//割り込み許可

    // IO割り込み http://www.arduino.cc/en/Reference/AttachInterrupt
    int CounterPin = 2 ;
    int CounterPinNo = 0 ;
    pinMode( CounterPin, INPUT ) ;
    digitalWrite( CounterPin, HIGH ) ;//turn on pullup resistors
    attachInterrupt( CounterPinNo, CountInterruptFunc, FALLING ) ;//LOW, CHANGE, RISING, FALLING

}

void asetup() {
    while( true ) {
        changeBaudrete(9600) ;
        for( int i = 0 ; i < 10 ; i ++ ) {
            btComm.println( "0000000000000");
            delay( 1000 ) ;
        }
        changeBaudrete(9600) ;
        for( int i = 0 ; i < 10 ; i ++ ) {
            btComm.println( "0000000000000");
            delay( 1000 ) ;
        }
    }
}
void setup() {
    initDebug();
    initTimerCounter();
    //最初に、初期化されているかどうかを、GEIGER_BAUD、AT+BTINFO?0 で確認する。
    changeBaudrete(GEIGER_BAUD);
    btComm.setTimeOut(2000,100);
    if( confirmBtConnection() == false ) {
        changeBaudrete(HARD_ORIGINAL_BAUD);
        confirmBtConnection();

        btComm.println("ATZ");//soft reset AT&Fコマンドの直前には必要なようだ
        replyPrint("1:ATZ",btComm.readString());
        delay(2000);btComm.readOut();

        btComm.println("AT&F");//工場出荷時に戻す
        replyPrint("2:AT&F",btComm.readString());
        delay(100);btComm.readOut();

        setupBluetoothModule() ;
        toScanMode();
        return ;
    }

    //モジュール名を比較して、初期化がちゃんとなっているかどうかを確認
    dbgLog.println("AT+BTINFO?0");
    btComm.println("AT+BTINFO?0");
    char* replyStr = btComm.readString();
    if( strcmp( replyStr, MACHINE_NAME ) != 0 ) {
        setupBluetoothModule() ;
    }
    toScanMode();
}
/**************************************************
  メインループ
**************************************************/
void loop() {
    while( 1 ) {
        btComm.setTimeOut( 10000, 300 ) ;
        char* buf = btComm.readString() ;
        if( buf == NULL ) {
            continue ;
        }
        else {
            if( strCmp( buf, "stt" ) == 0 ) {
                inMeasure = true ;
            }
            else if( strCmp( buf, "stp" ) == 0 ) {
                inMeasure = false ;
                btComm.println( "ok" ) ;
            }
            else if( strCmp( buf, "nop" ) == 0 ) {
                inMeasure = false ;
                btComm.println( "ok" ) ;
            }
            else if( strCmp( buf, "ver" ) == 0 ) {
                inMeasure = false ;
                btComm.println( "ver 0.00" ) ;
                btComm.println( "ok" ) ;
            }
            else if( strCmp( buf, "nam" ) == 0 ) {
                inMeasure = false ;
                btComm.println( "akiduki 000" ) ;
                btComm.println( "ok" ) ;
            }
            else if( strCmp( buf, "opt" ) == 0 ) {
                inMeasure = false ;
                btComm.println( "ok" ) ;
            }
            else {
                inMeasure = false ;
                //btComm.println( "error" ) ;//エラー
            }
        }
    }
}
void setupBluetoothModule( void )
{
    btComm.println("AT+BTUART," GEIGER_BAUD_STR ",N,1");//ATZの後にボーレートが変更されることに注意
    replyPrint("2:AT+BTUART," GEIGER_BAUD_STR ",N,1",btComm.readString());
    delay(100);btComm.readOut();

    btComm.println("ATZ");//soft reset AT&Fコマンドの直前には必要なようだ
    replyPrint("1:ATZ",btComm.readString());
    delay(2000);btComm.readOut();

    changeBaudrete(GEIGER_BAUD);
}
void toScanMode()
{
    btComm.println("ATZ");//ATZ の後に、PIN等を決めてやらないと駄目なようだ
    replyPrint("3:ATZ",btComm.readString());
    delay(1000);btComm.readOut();

    btComm.println("AT+BTKEY=1234");
    replyPrint("4:AT+BTKEY=1234",btComm.readString());
    delay(100);btComm.readOut();

    btComm.println("AT+BTSEC,0,0");
    replyPrint("5:AT+BTSEC",btComm.readString());
    delay(100);btComm.readOut();delay(500);

    btComm.println("AT+BTNAME=CaGeiger");
    replyPrint("6:AT+BTNAME=CaGeiger",btComm.readString());
    delay(100);btComm.readOut();delay(500);

    btComm.println("AT+BTINFO?0");
    replyPrint("7:AT+BTINFO?0",btComm.readString());
    delay(100);btComm.readOut();

    btComm.println("AT+BTINFO?1");
    replyPrint("8:AT+BTINFO?1",btComm.readString());
    delay(100);btComm.readOut();

    btComm.println("AT+BTSCAN");
    replyPrint("9:AT+BTSCAN",btComm.readString());
    delay(100);btComm.readOut();
}
void changeBaudrete(long baudrate)
{
    dbgLog.print( "changeBaudrete " ) ;
    dbgLog.println(baudrate,DEC);
    btComm.begin(baudrate);//
}
bool confirmBtConnection()
{
    dbgLog.println("confirmBtConnection");
    //Serial.begin()直後は、ゴミが入っていることが多いので、これをやっている
    for( int i = 0 ; i < 3 ; i ++ ) {
        btComm.println("AT");
        dbgLog.println("AT");
        char* replyStr = btComm.readString();
        dbgLog.print("[");dbgLog.print(replyStr);dbgLog.print("]");
        if( strcmp( replyStr, "OK" ) == 0 ) {
            delay(200);btComm.readOut();
            dbgLog.println("OK");
            return true ;
        }
    }
    dbgLog.println("NG");
    btComm.readOut();
    return false ;
}
void replyPrint( char* title, char* str )
{
    dbgLog.print(title);
    dbgLog.print(" ");
    replyPrint(str);
}
void replyPrint( char* str )
{
    dbgLog.print("reply[");
    dbgLog.print(str);
    dbgLog.println( "]" ) ;
}
int strCmp( char* dist, char* src )
{
    int len = strlen( src ) ;
    return strncmp( dist, src, len ) ;
}

